

#ifndef _Oz_Cms_CmsSite_ICE_
#define _Oz_Cms_CmsSite_ICE_

#ifndef INCLUDE_OZ_BASE_ONREZ
#define INCLUDE_OZ_BASE_ONREZ
	#include <Base/OnRez.ice>
#endif

#ifndef INCLUDE_OZ_DOMAINS_DOMAIN
#define INCLUDE_OZ_DOMAINS_DOMAIN
	#include <Domains/Domain.ice>
#endif

#ifndef INCLUDE_OZ_FILES_FILESYSTEM
#define INCLUDE_OZ_FILES_FILESYSTEM
	#include <Files/FileSystem.ice>
#endif

#ifndef INCLUDE_OZ_FILES_DOCUMENT
#define INCLUDE_OZ_FILES_DOCUMENT
	#include <Files/Document.ice>
#endif

#ifndef INCLUDE_OZ_DOMAINS_DOMAIN
#define INCLUDE_OZ_DOMAINS_DOMAIN
	#include <Domains/Domain.ice>
#endif

#ifndef INCLUDE_OZ_DOMAINS_GROUP
#define INCLUDE_OZ_DOMAINS_GROUP
	#include <Domains/Group.ice>
#endif

#ifndef INCLUDE_OZ_CMS_USERPROFILE
#define INCLUDE_OZ_CMS_USERPROFILE
	#include <Cms/UserProfile.ice>
#endif

#ifndef INCLUDE_OZ_CMS_CONFIRMATIONMAIL
#define INCLUDE_OZ_CMS_CONFIRMATIONMAIL
	#include <Cms/ConfirmationMail.ice>
#endif

#ifndef INCLUDE_OZ_MAIL_MAILACOUNT
#define INCLUDE_OZ_MAIL_MAILACOUNT
	#include <Mail/MailAcount.ice>
#endif

#ifndef INCLUDE_OZ_CMS_LOCALE
#define INCLUDE_OZ_CMS_LOCALE
	#include <Cms/Locale.ice>
#endif
module Oz
{
	module Cms
	{
		/**
		 * Estructura con los datos necesario para 
		 * crear una nueva cuenta de usuario.
		 */
		 
		struct AcountData
		{
			string name;
			string mail;
			string password;
			string locale;
		};

		dictionary<Ice::Identity,AcountData>AcountDataMap;

		interface CmsSiteClient
		{
		
			/**
			 * Inizicalizar el sitio cuando se crea
			 */
			 
			 idempotent void
			 initialize();
			/**
			 * Devuelve el nombre del sitio
			 */
			nonmutating string 
			getName();

			/**
			 * Devuelve la traducion del nombre para el
			 * locale que recive como parametro
			 */
			nonmutating string
			getTranslatedName(
				string locale);

			/**
			 * Permite establecer la traducion del nombre
			 * para un locale
			 */
			 
			idempotent void
			setTranslatedName(
				string locale,
				string name);

			/**
			 * Devuelve un proxy al sistema de ficheros que 
			 * usa el sitio.
			 */

			nonmutating Oz::Files::FileSystem* 
			getFileSystem();

			/**
			 * Devuelve la descripcion de este site
			 * para el locale especificado
			 */

			nonmutating string 
			getDescription(
				string locale);
				
			
			/**
			 * Escrive la propiedad description para le locale 
			 * especificado.
			 * ej: setDescription("es","Sitio web dedicado a");
			 */

			idempotent void setDescription(
				string locale,
				string description);
				
			/**
			 * Devuelve un proxy al Dominio que usa el site
			 * para gestionar sus usuarios.
			 */
				
			nonmutating Oz::Domains::Domain* getDomain();
			
			
			/**
			 * Permite crear una nueva cuenta en este sitio
			 * 
			 * @param name, nombre para el loging
			 * @param mail, direcion de correo a la que se envia el mensage
			 * de confirmacin.
			 * @param password, Password que utilzara el usuario para hacer
			 * login.
			 * @param locale, Locale asignado por defecto al usuario, este 
			 * se almacenara mas tarde en las propiedades de usuario.
			 */
			 
			idempotent void
			createAcount(
				string name,
				string mail,
				string password,
				string locale);
				
			/**
			 * Permite elminar una cuenta del site dado su nombre
			 *
			 * @param name, nombre del usuario del cual queremos borrar
			 * la cuenta.
			 */

			idempotent void
			deleteAcount(
				string name);

			/**
			 * Este metodo permite confirmar la creacion de una cuenta
			 *
			 * @param id, Id de confirmacion para la cuenta
			 * @param confirm, verdadero o false indica si se confirma o deniega
			 * la creacion de la cuenta.
			 *
			 * el id de confirmacion se envia al mail que especifico el usuario
			 * en createAcount
			 *
			 */
 
			idempotent bool
			confirmAcount(
				Ice::Identity id,
				bool confirm);
				
			/**
			 * Devuelve un proxy al perfil de usuario dado el id del usuario.
			 *
			 * @param userId, id del usuario que se quiere recuperar el perfil
			 * @returns UserProfile* prox al perfil del usuario o null si no
			 * existe dicho usuario.
			 *
			 */

			nonmutating UserProfile* 
			getUserProfile(Ice::Identity userId);
			
			/**
			 * permite comprovar si un login esta disponible para la creacion
			 * de una nueva cuenta en este site
			 */

			nonmutating bool
			isLoginAvailable(
				string login);
				
			/**
			 *
			 * permite comprovar si un email esta disponible para crear una
			 * nueva cuenta.
			 */
			 
			 nonmutating bool
			 isMailAvailable(
			 	string login);

			/**
			 * Permite recueperar el password de un usuario cuando lo olvida
			 * @param mail, cuenta de correo asociada al usuario que se quiere
			 * recuperar el password.
			 *
			 * Si existe un usuario con ese mail se envia un mensage con un link
			 * que permitira restablecer el password de usuario.
			 */
			 
			idempotent void
			restorePassword(
				string mail);
				
			/**
			 * Permite aadir un nuevo locale a la lista de locales del sitio
			 */

			idempotent Locale*
			addLocale(
				string countryCode);
				
			/**
			 * permite eliminar un locale
			 */

			idempotent void
			delLocale(string countryCode);

			/**
			 * Permite recuperar la lista de locales que usa el sitio
			 */

			nonmutating StringLocaleMap
			getLocales();

			/**
			 * Permite asignar el locale por defecto
			 */
			 
			idempotent void
			setDefaultLocale(
				string locale);

			/**
			 * Devuelve el locale por defecto para este sitio
			 */
			nonmutating string
			getDefaultLocale();
			
			/**
			 * Devuelve un proxy a la cuenta de correo
			 * usada por este servicio.
			 */

			nonmutating Oz::Mail::MailAcount*
			getMailAcount();

			nonmutating ConfirmationMail*
			getConfirmationMail();
			
			/**
			 * Devuelve el documento home de este sitio
			 */
			 
			nonmutating Oz::Files::Document*
			getHomeDocument();
		};

		class CmsSite 
			extends 
				Oz::Base::OnRez 
			implements 
				CmsSiteClient
		{
			/**
			 * Nombre del CmsSite debe es unico para todos
			 * los sites de un mismo CmsServer
			 *
			 */
			 
			string name;
			
			/**
			 * Traduciones del nombre 
			 */
			Oz::Base::StringMap translatedName;
			/**
			 * Descripcion basica del sitio
			 */
			Oz::Base::StringMap description;
			
			/**
			 * Sistema de ficheros que usa el sitio
			 */
			 
			Oz::Files::FileSystem* fileSystem;
			
			/**
			 * Dominio en el que se registran los usuarios del
			 * sitio.
			 */
			Oz::Domains::Domain* domainPx;
			
			/**
			 * Grupos de usuarios que indentifica
			 * los distintos roles de un sitio.
			 * Admininistradores, Editores, Usuarios resgistrados,
			 * Usuarios sin registrar respectivamente
			 */
			Oz::Domains::Group* adminsGroup;
			Oz::Domains::Group* editorsGroup;
			Oz::Domains::Group* usersGroup;
			Oz::Domains::Group* nobodyGroup;
			
			/**
			 * Locale por defecto usado para el sitio.
			 */
			string defaultLocale;
			
			/**
			 * Lista de locales para el sitio
			 */
			StringLocaleMap locales;
			
			/**
			 * Cuenta de correo utilizada para enviar 
			 * notificaciones.
			 */
			Oz::Mail::MailAcount* mailAcount;
			
			/**
			 * Mail que se envia a los nuevos usuarios
			 */
			Oz::Cms::ConfirmationMail* confirmationMail;
			
			/**
			 * Perfiles de usuarios
			 */
			UserProfileMap profilesMap;
			
			/**
			 * Mapa con las cuentas pendientes de activacion
			 */
			AcountDataMap acountsToActive;
			Oz::Files::Document* homeDocument;
		};

		["java:type:java.util.TreeMap"]
			dictionary <string,CmsSite*> CmsSiteMap;
	};
};

#endif
