
/***************************************************************************/
/*Copyright (C)  On-Rez                                                   **/
/*www.on-rez.com                                                          **/
/*                                                                        **/
/*This code is licensed to you under the terms of GPL license             **/
/*a copy ot it is available with the sources in the file LICENSE.         **/
/*you can view online at http://www.gnu.org/licenses/gpl.txt              **/
/***************************************************************************/

#ifndef _Oz_Domains_Domain_ICE_
#define _Oz_Domains_Domain_ICE_

//Oz includes

#ifndef INCLUDE_OZ_BASE_ONREZ
#define INCLUDE_OZ_BASE_ONREZ
	#include <Base/OnRez.ice>
#endif

#ifndef INCLUDE_OZ_DOMAINS_USER
#define INCLUDE_OZ_DOMAINS_USER
	#include <Domains/User.ice>
#endif

#ifndef INCLUDE_OZ_DOMAINS_GROUP
#define INCLUDE_OZ_DOMAINS_GROUP	
	#include <Domains/Group.ice>
#endif

#ifndef INCLUDE_OZ_FILES_FILESYSTEM
#define INCLUDE_OZ_FILES_FILESYSTEM
	#include <Files/FileSystem.ice>
#endif

#ifndef INCLUDE_OZ_FILES_FILE
#define INCLUDE_OZ_FILES_FILE
	#include <Files/File.ice>
#endif

module Oz
{
	module Domains
	{
		interface DomainClient
		{
			idempotent User* login(string name,string passwd);
		};
	
		interface DomainAdmin
		{
			nonmutating string 
			getName();
	
			idempotent User* 
			createUser(
				string name,
				string passwd,
				string mail);
			
			idempotent bool 
			deleteUser(
				Ice::Identity userId);

			nonmutating User* 
			getUserId(
				Ice::Identity id);

			nonmutating User*
			getUserNamed(string name);

			nonmutating Group* 
			getGroupId(
				Ice::Identity id);
				
			nonmutating UsersMap 
			getUsers();
			
			nonmutating UsersMap 
			getUsersId(
				Oz::Base::Identities ids);

			idempotent Group* 
			createGroup(
				string name);
			
			idempotent bool 
			deleteGroup(
				Ice::Identity groupId);
	
			nonmutating GroupsMap 
			getGroups();
			
			nonmutating GroupsMap 
			getGroupsId(
				Oz::Base::Identities ids);
			
			nonmutating Group* 
			getGroupName(
				string name);
		
			nonmutating Oz::Base::EntidadIdMap 
			getEntidades();
	
			idempotent bool 
			addUserToGroup(
				Ice::Identity usuario,
				Ice::Identity group);
			
			idempotent bool 
			delUserToGroup(
				Ice::Identity usuario,
				Ice::Identity group);
				
			idempotent bool 
			setFileSystem(
				Oz::Files::FileSystem* fileSys);
			
			nonmutating Oz::Files::FileSystem* 
			getFileSystem();
			
			nonmutating Oz::Files::File* 
			getIcon();
		};
		
		class Domain 
			extends 
				Oz::Base::OnRez 
			implements 
				DomainAdmin,
				DomainClient
		{
				string domainName;
				UsersMap users;
				Oz::Base::StringIdMap loginsMap;
				GroupsMap grupos;
				Oz::Base::StringIdMap groupMap;

				Oz::Files::FileSystem* fileSystem;
				Oz::Files::File* icon;
		};

		["java:type:java.util.TreeMap"]
			dictionary<string,Domain*> DomainNameMap;
	};
};
#endif

