
/***************************************************************************/
/*Copyright (C)  On-Rez                                                   **/
/*www.on-rez.com                                                          **/
/*                                                                        **/
/*This code is licensed to you under the terms of GPL license             **/
/*a copy ot it is available with the sources in the file LICENSE.         **/
/*you can view online at http://www.gnu.org/licenses/gpl.txt              **/
/***************************************************************************/

#ifndef _Oz_Files_FileSystem_ICE_
#define _Oz_Files_FileSystem_ICE_

//Oz includes

#ifndef INCLUDE_OZ_BASE_ONREZ
#define INCLUDE_OZ_BASE_ONREZ
	#include <Base/OnRez.ice>
#endif

#ifndef INCLUDE_OZ_FILES_DIRECTORY
#define INCLUDE_OZ_FILES_DIRECTORY
	#include <Files/Directory.ice>
#endif

#ifndef INCLUDE_OZ_FILES_LINK
#define INCLUDE_OZ_FILES_LINK
	#include <Files/Link.ice>
#endif

#ifndef INCLUDE_OZ_BASE_CACHEMAP
#define INCLUDE_OZ_BASE_CACHEMAP
	#include <Base/CacheMap.ice>
#endif

module Oz
{
	module Files
	{	
		interface FileSystemWriter
		{
			idempotent bool 
			setName(
				string name);
			
			idempotent bool 
			setRootDir(
				Directory* rootDir);
				
			/**
			 * Permite crear un nuevo nodo en el sistema de ficheros
			 *
			 * @param type, el tipo de nodo que queremos crear 
			 * ("Directory","File","Document","Image")
			 * @param name, el nombre que se asignara al nuevo nodo
			 *
			 * @param path, ruta en la que se crea el nuevo nodo
			 *
			 * @returns Node*, proxy al nuevo nodo 
			 */

			idempotent Node* 
			createNode(
				string type,
				string name,
				string path);

			idempotent Link*
			createLink(
				string name,
				string sourcePath,
				string targetPath);
			
			idempotent bool 
			deleteNode(
				Node* nodo);

			idempotent bool
			copyNode(
				Node* sourceNode,
				string targetPath,
				FileSystem* targeFileSystem);

			idempotent bool
			moveNode(
				Node* sourceNode,
				string targetPath,
				FileSystem* targeFileSystem);

			idempotent void 
			nodeChanged(
				Node* nodo);

			idempotent void 
			setChangesCacheSize(
				long size);

			nonmutating long
			geChangesCacheSize();


			idempotent void 
			setChangesCache(
				Oz::Base::CacheMap* cache);

			nonmutating Oz::Base::ObjectPosMap
			getChangesCacheMap();
		};
		
		interface FileSystemReader
		{
			/**
			 * Nombre asignado al sistema de ficheros
			 */
			nonmutating string 
			getName();
			
			/**
			 * Permite recuperar un proxy a un nodo dado su 
			 * path.
			 */
			nonmutating Node* 
			getNode(
				string path);
		};
		
		/**
		 * Esta clase represneta un sistema de ficheros
		 * 
		 */
		class FileSystem 
			extends 
				Oz::Base::OnRez 
			implements 
				FileSystemWriter,
				FileSystemReader
		{
			/**
			 * Nombre del sistema de ficheros.
			 */
			
			string name;
			
			/*
			 * Proxy al directorio raiz del sistema
			 * de ficheros.
			 */
			Directory* rootDir;
			
			/**
			 * Cache con proxys a los nodos modificados
			 * mas recientemente.
			 */
			Oz::Base::CacheMap* cache;
		};
	};
};
#endif

