
/***************************************************************************/
/*Copyright (C)  On-Rez                                                   **/
/*www.on-rez.com                                                          **/
/*                                                                        **/
/*This code is licensed to you under the terms of GPL license             **/
/*a copy of it is available with the sources in the file LICENSE.         **/
/*you can view online at http://www.gnu.org/licenses/gpl.txt              **/
/***************************************************************************/

#ifndef _Oz_Shops_Order_ICE_
#define _Oz_Shops_Order_ICE_

//Oz includes

#ifndef INCLUDE_OZ_BASE_ONREZ
#define INCLUDE_OZ_BASE_ONREZ
	#include <Base/OnRez.ice>
#endif

#ifndef INCLUDE_OZ_BASE_DATE
#define INCLUDE_OZ_BASE_DATE
	#include <Base/Date.ice>
#endif

#ifndef INCLUDE_OZ_PRODUCTS_PRODUCT
#define INCLUDE_OZ_PRODUCTS_PRODUCT
	#include <Products/Product.ice>
#endif

#ifndef INCLUDE_OZ_SHOPS_LINEITEM
#define INCLUDE_OZ_SHOPS_LINEITEM
	#include <Shops/LineItem.ice>
#endif

#ifndef INCLUDE_OZ_PRICES_PRICELIST
#define INCLUDE_OZ_PRICES_PRICELIST
	#include <Prices/PriceList.ice>
#endif

#ifndef INCLUDE_OZ_DOMAINS_USER
#define INCLUDE_OZ_DOMAINS_USER
	#include <Domains/User.ice>
#endif

#ifndef INCLUDE_OZ_ADDRESSES_ADDRESS
#define INCLUDE_OZ_ADDRESSES_ADDRESS
	#include <Addresses/Address.ice>
#endif

module Oz
{
	module Shops
	{
		class Pos;
		class Space;

		interface OrderAdmin
		{
			idempotent bool 
			setPriceList(
				Oz::Prices::PriceList* priceList);
				
			idempotent LineItem* 
			addToOrder(
				Oz::Products::Product* product,
				int units);
			
			idempotent LineItem* 
			delFromOrder(
				Oz::Products::Product* product,
				int units);

			idempotent bool 
			confirm();
			
			idempotent bool 
			annul();
		};
		
		interface OrderClient
		{
			nonmutating Oz::Base::Date 
			getCreationDate();
			
			nonmutating Oz::Base::Date 
			getCloseDate();
			
			nonmutating LineItemMap 
			getItems();
			
			nonmutating bool 
			isClosed();
			
			nonmutating bool 
			isOpened();
			
			nonmutating bool 
			isAnnuled();
			
			nonmutating double 
			getTotal();
			
			nonmutating Oz::Domains::User* 
			getClient();
			
			nonmutating Oz::Addresses::Address
			getAdressToSend();
		};
		
		class Order 
			extends 
				Oz::Base::OnRez 
			implements 
				OrderAdmin,
				OrderClient
		{
			Oz::Base::Date creationDate;
			Oz::Base::Date closeDate;
			Oz::Domains::User* client;
			Oz::Addresses::Address addressToSend;
			bool closed;
			bool annuled;
			bool confirmed;
			bool send;
			Oz::Prices::PriceList* priceList;
			
			/**
			 * Here we asociated products identity
			 * with the correpondent LineItem proxy
			 */
			
			LineItemMap products;
		};
		
		dictionary<Ice::Identity,Order*> OrderMap;
	};
};
#endif

