
/***************************************************************************/
/*Copyright (C)  On-Rez                                                   **/
/*www.on-rez.com                                                          **/
/*                                                                        **/
/*This code is licensed to you under the terms of GPL license             **/
/*a copy of it is available with the sources in the file LICENSE.         **/
/*you can view online at http://www.gnu.org/licenses/gpl.txt              **/
/***************************************************************************/

#ifndef _Oz_Shops_Shop_ICE_
#define _Oz_Shops_Shop_ICE_

#ifndef INCLUDE_ICE_IDENTITY
#define INCLUDE_ICE_IDENTITY
	#include <Ice/Identity.ice>
#endif

#ifndef INCLUDE_OZ_BASE_ONREZSERVICE
#define INCLUDE_OZ_BASE_ONREZSERVICE
	#include <Base/OnRezService.ice>
#endif

#ifndef INCLUDE_OZ_DOMAINS_DOMAIN
#define INCLUDE_OZ_DOMAINS_DOMAIN
	#include <Domains/Domain.ice>
#endif

#ifndef INCLUDE_OZ_PRODUCT_PRODUCTCATEGORY
#define INCLUDE_OZ_PRODUCT_PRODUCTCATEGORY
	#include <Products/ProductCategory.ice>
#endif

#ifndef INCLUDE_OZ_ADDRESSES_ADDRESSBOOK
#define INCLUDE_OZ_ADDRESSES_ADDRESSBOOK
	#include <Addresses/AddressBook.ice>
#endif

#ifndef INCLUDE_OZ_ADDRESSES_CORPORATION
#define INCLUDE_OZ_ADDRESSES_CORPORTATON
	#include <Addresses/Corporation.ice>
#endif

#ifndef INCLUDE_OZ_SHOPS_ORDER
#define INCLUDE_OZ_SHOPS_ORDER
	#include <Shops/Order.ice>
#endif

#ifndef INCLUDE_OZ_PRICES_PRICESERVER
#define INCLUDE_OZ_PRICES_PRICESERVER
	#include <Prices/PriceServer.ice>
#endif

module Oz
{
	module Shops
	{
		interface ShopAdmin
		{	
			idempotent bool 
			setDescription(
				string description);
			
			idempotent Order* createBasket(Ice::Identity userId);
			idempotent void destroyBasket(Ice::Identity basketId);
			idempotent Order* purchaseOrder(Ice::Identity orderId);
			
			idempotent void 
			deleteOrder(
				Ice::Identity id);
		};
		
		interface ShopClient
		{	
			nonmutating Oz::Products::ProductCategory* 
			getCatalog();
			
			nonmutating Oz::Prices::PriceServer* 
			getPriceServer();
			
			nonmutating Oz::Domains::Domain* 
			getDomain();
			
			nonmutating string 
			getName();
			
			nonmutating Oz::Addresses::AddressBook* 
			getAddressBook();
			
			nonmutating Oz::Addresses::Corporation* 
			getCorporation();
			
			nonmutating string 
			getDescription();
		};
		
		class Shop 
			extends 
				Oz::Base::OnRezService
			implements 
				ShopAdmin,
				ShopClient
		{
			string name;
			string description;
			Oz::Domains::Domain* domain;
			Oz::Addresses::AddressBook* addressBook;
			Oz::Addresses::Corporation* corporation;
			
			Oz::Products::ProductCategory* products;
			Oz::Prices::PriceServer* priceLists;
			
			OrderMap orders;
			OrderMap ordersByUser;
			OrderMap baskets;
		};
	};
};
#endif

